package ru.dbotthepony.kstarbound.world.entities.behavior

import ru.dbotthepony.kstarbound.defs.actor.behavior.BehaviorDefinition
import ru.dbotthepony.kstarbound.defs.actor.behavior.NodeParameterValue
import ru.dbotthepony.kstarbound.lua.userdata.BehaviorState

class BehaviorTree(val blackboard: Blackboard, val data: BehaviorDefinition, overrides: Map<String, NodeParameterValue> = mapOf()) : AbstractBehaviorNode() {
	val scripts = HashSet(data.scripts)
	val functions = HashSet<String>()

	val root: AbstractBehaviorNode

	init {
		if (overrides.isEmpty()) {
			root = create(data.root, data.mappedParameters, this)
		} else {
			val parameters = LinkedHashMap(data.mappedParameters)
			parameters.putAll(overrides)
			root = create(data.root, parameters, this)
		}
	}

	override fun run(delta: Double, state: BehaviorState): Status {
		return root.run(delta, state)
	}

	override fun reset() {
		root.reset()
	}
}

